// START A3HEADER
//
// The Atlantis program, and all of it's source code (including
// this file) is Copyright (c) 1995-1998 by Geoff Dunbar. This
// source code is distributed under the terms of the license
// agreement in the file LICENSE.txt. If this file was not included
// with the source code you received, you may not use, modify, or
// distribute this source code.
//
// See the Atlantis Project web page for details:
// http://www.prankster.com/project
//
// END A3HEADER
// This file contains extra game-specific functions
//
#include "game.h"
#include "rules.h"

int Game::SetupFaction( Faction *pFac )
{
    if( TurnNumber() != 1 )
    {
        //
        // Factions can only be added during the beginning of the game.
        //
        return( 0 );
    }

    //
    // Let's see if there is an available region for this unit.
    //
    ARegion *pReg = 0;
    forlist( &regions ) {
        pReg = (ARegion *) elem;
        ARegionArray *pRA = regions.pRegionArrays[ pReg->zloc ];

        if( !pReg->CanBeStartingCity( pRA ) )
        {
            pReg = 0;
            continue;
        }

        if( pReg->IsStartingCity() )
        {
            //
            // This region has already been set up.
            //
            pReg = 0;
            continue;
        }

        break;
    }

    if( !pReg )
    {
        //
        // We couldn't find a region to make this faction's unit in.
        //
        return( 0 );
    }

    pFac->unclaimed = Globals->START_MONEY + TurnNumber() * 50;

    //
    // Make a citadel for this faction.
    //
    Object *obj = new Object( pReg );
    obj->num = pReg->buildingseq++;
    obj->name = new AString( AString( "Citadel [" ) + obj->num + "]" );
    obj->type = O_CITADEL;
    obj->incomplete = 0;
    obj->inner = -1;
    pReg->objects.Add( obj );

    //
    // Set up first unit.
    //
    Unit *temp2 = GetNewUnit( pFac );
    temp2->SetMen( I_MAN, 1 );
    temp2->reveal = REVEAL_FACTION;
    temp2->MoveUnit( obj );

    return( 1 );
}

void Game::PostProcessUnitExtra(ARegion *r,Unit *u)
{
    //
    // Nothing to do
    //
}

void Game::CheckUnitMaintenance( int consume )
{
    //
    // Nothing to do
    //
}

void Game::CheckFactionMaintenance( int consume )
{
    //
    // Nothing to do
    //
}

void Game::CheckAllyMaintenance()
{
    //
    // Nothing to do
    //
}

Faction *Game::CheckVictory()
{
    Faction *pVictor = 0;

    //
    // First, if there is only one living faction, he is the winner.
    //
    {
        forlist( &factions ) {
            Faction *pFac = (Faction *) elem;
            if( pFac->exists )
            {
                if( pVictor )
                {
                    //
                    // This is the second faction we've found; that means
                    // no-one has won due to being the only faction.
                    //
                    pVictor = 0;
                    break;
                }
                pVictor = pFac;
            }
        }
    }
    if( pVictor )
    {
        return( pVictor );
    }

    //
    // Next check: if one faction holds all of the citadels, he is the
    // winner.
    //
    {
        forlist( &regions ) {
            ARegion *pReg = (ARegion *) elem;
            
            if( pReg->IsStartingCity() )
            {
                forlist( &( pReg->objects )) {
                    Object *pObj = (Object *) elem;
                    
                    if( pObj->type != O_CITADEL )
                    {
                        continue;
                    }
                    
                    Unit *u = pObj->GetOwner();
                    
                    if( !u )
                    {
                        //
                        // No one owns this citadel.
                        //
                        return( 0 );
                    }
                    
                    if( !pVictor )
                    {
                        pVictor = u->faction;
                    }
                    else
                    {
                        if( pVictor != u->faction )
                        {
                            return( 0 );
                        }
                    }
                    
                    //
                    // We already found one citadel; no need to continue with
                    // this object.
                    //
                    break;
                }
            }
        }
    }

    return( pVictor );
}

int Game::AllowedMages( Faction *pFac )
{
    return( 5 );
}

int Game::AllowedTaxes( Faction *pFac )
{
    return( -1 );
}

int Game::AllowedTrades( Faction *pFac )
{
    return( -1 );
}

int Unit::GetObservationBonus()
{
    int bonus = 0;
    if (GetMen()) 
    {
        bonus = GetSkill( S_TRUE_SEEING );
        if( bonus < 3 && items.GetNum( I_AMULET_OF_TRUE_SEEING ))
        {
            bonus = 3;
        }
    }
    return( bonus );
}

int Unit::GetStealthBonus()
{
    int bonus = 0;
    int men = GetMen();
    if( men == 1 )
    {
        bonus = GetSkill( S_INVISIBILITY );
    }
    if( bonus < 3 )
    {
        if( men <= items.GetNum( I_AMULET_OF_INVISIBILITY ))
        {
            bonus = 3;
        }
    }
    return( bonus );
}
