// START A3HEADER
//
// The Atlantis program, and all of it's source code (including
// this file) is Copyright (c) 1995-1998 by Geoff Dunbar. This
// source code is distributed under the terms of the license
// agreement in the file LICENSE.txt. If this file was not included
// with the source code you received, you may not use, modify, or
// distribute this source code.
//
// See the Atlantis Project web page for details:
// http://www.prankster.com/project
//
// END A3HEADER
#include "gamedefs.h"
#include "game.h"
#include "items.h"
#include "skills.h"
#include "rules.h"

void usage()
{
    Awrite("atlantis new <gamefile> <playerfile>");
    Awrite("");
    Awrite("atlantis map <gamefile> <type> <mapfile>");
    Awrite("");
    Awrite("atlantis run <ingame> <admin> <orders> <reports> <outgame> "
           "<playerfile> <times header>");
}

void CheckOneAbbr(char * abr) {
  int i;
  int c = 0;
  AString temp;

  for (i=0; i<NITEMS; i++) {
    if (AString(abr) == AString(ItemDefs[i].abr)) {
      c ++;
      temp += AString(abr) + " : item  ";
    }
  }

  for (i=0; i<NSKILLS; i++) {
    if (AString(abr) == AString(SkillDefs[i].abbr)) {
      c ++;
      temp += AString(abr) + " : skill  ";
    }
  }

  if (c>1) {
    Awrite(temp);
  }
}

void CheckAbbrs()
{
    Awrite("Checking Abbreviations:");
    
    int i;
    for (i=0; i<NITEMS; i++) {
        CheckOneAbbr(ItemDefs[i].abr);
    }
    
    for (i=0; i<NSKILLS; i++) {
        CheckOneAbbr(SkillDefs[i].abbr);
    }
}

int main(int argc, char *argv[])
{
    Game game;

    initIO();

    Awrite( AString( "Atlantis Engine Version: " ) + 
            ATL_VER_STRING( CURRENT_ATL_VER ));
    Awrite( AString( Globals->RULESET_NAME ) + ", Version: " +
            ATL_VER_STRING( Globals->RULESET_VERSION ));
    Awrite( "" );


#ifdef NEVER
    CheckAbbrs();
#endif
    
    if (argc == 1)
    {
        usage();
        
        doneIO();
        return 0;
    }

    do
    {
        if (AString(argv[1]) == "new")
        {
            if( !game.NewGame() )
            {
                Awrite( "Couldn't make the new game!" );
                break;
            }

            if( !game.SaveGame() )
            {
                Awrite( "Couldn't save the game!" );
                break;
            }

            if( !game.WritePlayers() )
            {
                Awrite( "Couldn't write the players file!" );
                break;
            }
        }

        if (AString(argv[1]) == "map")
        {
            if (argc == 4)
            {
                if( !game.OpenGame() )
                {
                    Awrite( "Couldn't open the game file!" );
                    break;
                }

                if( !game.ViewMap( argv[2], argv[3] ))
                {
                    Awrite( "Couldn't write the map file!" );
                    break;
                }
            }
            else
            {
                usage();
                break;
            }
        }

        if (AString(argv[1]) == "run")
        {
            if( !game.OpenGame() )
            {
                Awrite( "Couldn't open the game file!" );
                break;
            }

            if( !game.RunGame() )
            {
                Awrite( "Couldn't run the game!" );
                break;
            }

            if( !game.SaveGame() )
            {
                Awrite( "Couldn't save the game!" );
                break;
            }
        }

        if (AString(argv[1]) == "edit")
        {
            if( !game.OpenGame() )
            {
                Awrite( "Couldn't open the game file!" );
                break;
            }

            int saveGame = 0;
            if( !game.EditGame( &saveGame ) )
            {
                Awrite( "Couldn't edit the game!" );
                break;
            }

            if( saveGame )
            {
                if( !game.SaveGame() )
                {
                    Awrite( "Couldn't save the game!" );
                    break;
                }
            }
        }

        if( AString( argv[1] ) == "check" )
        {
            if( argc == 4 )
            {
                game.DummyGame();
                if( !game.DoOrdersCheck( argv[ 2 ], argv[ 3 ] ))
                {
                    Awrite( "Couldn't check the orders!" );
                    break;
                }
            }
            else
            {
                usage();
                break;
            }
        }
    }
    while( 0 );

    doneIO();
    return 0;
}




