// START A3HEADER
//
// The Atlantis program, and all of it's source code (including
// this file) is Copyright (c) 1995-1998 by Geoff Dunbar. This
// source code is distributed under the terms of the license
// agreement in the file LICENSE.txt. If this file was not included
// with the source code you received, you may not use, modify, or
// distribute this source code.
//
// See the Atlantis Project web page for details:
// http://www.prankster.com/project
//
// END A3HEADER
#include "market.h"
#include "items.h"
#include "gameio.h"
#include "rules.h"

Market::Market() {
  activity = 0;
}

Market::Market(int a,int b,int c,int d,int e,int f,int g,int h) {
  type = a;
  item = b;
  price = c;
  amount = d;

  minpop = e;
  maxpop = f;
  minamt = g;
  maxamt = h;

  baseprice = price;
  activity = 0;
}

void Market::PostTurn(int population,int wages)
{
    if( !( Globals->VARIABLE_ECONOMY ))
    {
        //
        // Nothing to do to the markets.
        //
        return;
    }

    //
    // Unlimited, unchanging market.
    //
    if (amount == -1) return;

    if (ItemDefs[item].type & IT_MAN)
    {
        if (item == I_LEADERS)
        {
            amount = population / 25;
            price = wages * 8;
        } 
        else
        {
            amount = population / 5;
            price = wages * 4;
        }
        return;
    }

    int tarprice = price;
    if (amount)
    {
        if (type == M_BUY)
        {
            tarprice = (2 * baseprice + (baseprice * activity) / amount) / 2;
        } 
        else
        {
            tarprice = (3 * baseprice - (baseprice * activity) / amount) / 2;
        }
    }
    price = price + (tarprice - price) / 5;

    if (population <= minpop)
    {
        amount = minamt;
    } 
    else
    {
        if (population >= maxpop)
        {
            amount = maxamt;
        }
        else
        {
            amount = minamt +
                ((maxamt - minamt) * (population - minpop)) /
                (maxpop - minpop);
        }
    }
}

void Market::Writeout(Aoutfile * f) {
#ifdef DEBUG_GAME
  f->PutStr("Market");
#endif
  f->PutInt(type);
  f->PutInt(item);
  f->PutInt(price);
  f->PutInt(amount);
  f->PutInt(minpop);
  f->PutInt(maxpop);
  f->PutInt(minamt);
  f->PutInt(maxamt);
  f->PutInt(baseprice);
}

void Market::Readin(Ainfile * f) {
#ifdef DEBUG_GAME
  delete f->GetStr();
#endif
  type = f->GetInt();
  item = f->GetInt();
  price = f->GetInt();
  amount = f->GetInt();
  minpop = f->GetInt();
  maxpop = f->GetInt();
  minamt = f->GetInt();
  maxamt = f->GetInt();
  baseprice = f->GetInt();
/* XXXXX - Take this out!!! */
#ifdef NEVER
  if (ItemDefs[item].type & IT_TRADE && amount >= 0) {
    minamt *= 10;
    maxamt *= 10;
    amount *= 10;
  }
#endif
}

AString Market::Report() {
  AString temp;
  temp += ItemString(item,amount) + " at $" + price;
  return temp;
}

void MarketList::PostTurn(int population,int wages) {
  forlist(this) {
    ((Market *) elem)->PostTurn(population,wages);
  }
}

void MarketList::Writeout(Aoutfile * f) {
#ifdef DEBUG_GAME
  f->PutStr("MarketList");
#endif
  f->PutInt(Num());
  forlist (this)
    ((Market *) elem)->Writeout(f);
}

void MarketList::Readin(Ainfile * f) {
#ifdef DEBUG_GAME
  delete f->GetStr();
#endif
  int n = f->GetInt();
  for (int i=0; i<n; i++) {
    Market * m = new Market;
    m->Readin(f);
    Add(m);
  }
}
