// START A3HEADER
//
// The Atlantis program, and all of it's source code (including
// this file) is Copyright (c) 1995-1998 by Geoff Dunbar. This
// source code is distributed under the terms of the license
// agreement in the file LICENSE.txt. If this file was not included
// with the source code you received, you may not use, modify, or
// distribute this source code.
//
// See the Atlantis Project web page for details:
// http://www.prankster.com/project
//
// END A3HEADER
#include "game.h"
#include "rules.h"

void Game::CreateCityMons()
{
    if( !Globals->CITY_MONSTERS_EXIST )
    {
        return;
    }

    forlist(&regions) {
        ARegion * r = (ARegion *) elem;
        if (r->type == R_NEXUS || r->IsStartingCity() || r->town)
        {
            CreateCityMon( r, 100 );
        }
    }
}

void Game::CreateWMons()
{
    if( !Globals->WANDERING_MONSTERS_EXIST )
    {
        return;
    }

    GrowWMons( 50 );
}

void Game::CreateLMons()
{
    if( !Globals->LAIR_MONSTERS_EXIST )
    {
        return;
    }

    GrowLMons( 50 );
}

void Game::GrowWMons(int rate)
{
    //
    // Now, go through each 8x8 block of the map, and make monsters if
    // needed.
    //
    int level;
    for( level = 0; level < regions.numLevels; level++ )
    {
        ARegionArray *pArr = regions.pRegionArrays[ level ];
        int xsec;
        for (xsec=0; xsec< pArr->x / 8; xsec++)
        {
            for (int ysec=0; ysec< pArr->y / 16; ysec++)
            {
                /* OK, we have a sector. Count mons, and wanted */
                int mons=0;
                int wanted=0;
                for (int x=0; x<8; x++)
                {
                    for (int y=0; y<16; y+=2)
                    {
                        ARegion *reg = pArr->GetRegion(x+xsec*8,y+ysec*16+x%2);
                        if (!reg->IsGuarded())
                        {
                            mons += reg->CountWMons();
                            wanted += TerrainDefs[reg->type].wmonfreq;
                        }
                    }
                }

                wanted /= 10;
                wanted -= mons;
                wanted = (wanted*rate + getrandom(100))/100;
                if (wanted > 0)
                {
                    for (int i=0; i< wanted;)
                    {
                        int m=getrandom(8);
                        int n=getrandom(8)*2+m%2;
                        ARegion *reg = pArr->GetRegion(m+xsec*8,n+ysec*16);
                        if (!reg->IsGuarded() && MakeWMon( reg ))
                        {
                            i++;
                        }
                    }
                }
            }
        }
    }
}

void Game::GrowLMons( int rate )
{
    forlist(&regions) {
        ARegion * r = (ARegion *) elem;
        //
        // Don't make lmons in guarded regions
        //
        if (r->IsGuarded()) continue;
        
        forlist(&r->objects) {
            Object * obj = (Object *) elem;
            if (obj->units.Num()) continue;
            int montype = ObjectDefs[obj->type].monster;
            if (montype != -1)
            {
                if (getrandom(100) < rate)
                {
                    MakeLMon( obj );
                }
            }
        }
    }
}









