// START A3HEADER
//
// The Atlantis program, and all of it's source code (including
// this file) is Copyright (c) 1995-1998 by Geoff Dunbar. This
// source code is distributed under the terms of the license
// agreement in the file LICENSE.txt. If this file was not included
// with the source code you received, you may not use, modify, or
// distribute this source code.
//
// See the Atlantis Project web page for details:
// http://www.prankster.com/project
//
// END A3HEADER
#ifndef SKILL_CLASS
#define SKILL_CLASS

class Skill;
class SkillList;

#include "fileio.h"
#include "astring.h"
#include "gamedefs.h"
#include "alist.h"

/* For dependencies:
  A value of depend == -1 indicates no more dependencies.
  If depend is set to a skill, to study this skill, you must know
  the depended skill at level equal to (at least) the level in the
  structure, or at the level you are trying to study to.

  Example:
  SANDLE has depend1 = SHOE and level1 = 3.

  To study:   requires:
  SANDLE 1    SHOE 3
  SANDLE 2    SHOE 3
  SANDLE 3    SHOE 3
  SANDLE 4    SHOE 4
  SANDLE 5    SHOE 5
*/
class SkillType
{
public:
    char * name;
    char * abbr;
    int cost;
    
    enum {
        MAGIC = 0x1,
        COMBAT = 0x2,
        CAST = 0x4,
        FOUNDATION = 0x8,
    };
    int flags;

    //
    // special for combat spells only
    //
    int special;

    int depend1;
    int level1;
    int depend2;
    int level2;
    int depend3;
    int level3;
};

extern SkillType * SkillDefs;

int ParseSkill(AString *);

AString SkillStrs(int);

class ShowType { 
public:
  int skill;
  int level;
  char * desc;
};

extern ShowType * ShowDefs;

int SkillCost(int);
int SkillMax(int,int); /* skill, race */

int GetLevelByDays(int);
int GetDaysByLevel(int);

class ShowSkill : public AListElem {
public:
  ShowSkill(int,int);
  
  AString * Report();
  
  int skill;
  int level;
};

class Skill : public AListElem {
public:
  void Readin(Ainfile *);
  void Writeout(Aoutfile *);
  
  Skill * Split(int,int); /* total num, num leaving */
	
  int type;
  int days;
};

class SkillList : public AList {
public:
  int GetDays(int); /* Skill */
  void SetDays(int,int); /* Skill, days */
  void Combine(SkillList *);
  SkillList * Split(int,int); /* total men, num to split */
  AString Report(int); /* Number of men */
  void Readin(Ainfile *);
  void Writeout(Aoutfile *);
};

#endif
